/* Copyright 2024 by COG9 Enterprises LLC
 * All rights reserved.
 * Title: buildindex.html
 * Author: Jed Marti
 * Description: Build index.html in .meshmail
 * Created:
 * Revision History:
 *   Mon Dec 30 20:47:21 2024 - JBM Code review.
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <dirent.h>

#include "meshmail.h"

// Where should mail go/come from. See meshmail.h for default, cmdline.c for setting.
extern char *MailDir;



void buildindex(int32_t nm)
/*     Build index.html in the mail directory. We examine all the files with numeric only
 *   names and build an index entry for them. Clicking on that line takes you to the
 *   message file. Not very fancy but cheap.
 * Parameters:
 *   nm: The number of new messages.
 * External References:
 *   MailDir: Where the message files sit.
 * Returned values: none
 * Errors detected: can't create file.
 */
{
  FILE *fh, *mfh;
  char fname[MAXPATHLEN];
  DIR  *dh;
  struct dirent *dent;
  int32_t i, c, n;

  // Where we're going to put the new message file.
  snprintf(fname, MAXPATHLEN, "%s/index.html", MailDir);
  if ((fh = fopen(fname, "wb")) == NULL)
  {
    fprintf(stderr, "%s:%s:%d Couldn't open %s for output because %s\n", __FILE__, __func__,
            __LINE__, fname, strerror(errno));
    exit(-1);
  }

  // Open the mail directory so we can read all the file names.
  if ((dh = opendir(MailDir)) == NULL)
  {
    fprintf(stderr, "%s:%s:%d Couldn't open mail directory %s because %s\n", __FILE__, __func__,
            __LINE__, MailDir, strerror(errno));
    exit(-1);
  }
    
  // Prolog.
  fprintf(fh, "<html>\n<head>\n<title> Messages</title>\n<body>\n");
  fprintf(fh, "<H1> KI7NNP Mail Service V%d.%02d.%03d</H1>\n",
          VERSION, RELEASE, PATCH);
  fprintf(fh, "<H1> Messages as of %s</H1>\n", date());

  // Header if we have new lists.
  if (nm == 1) fprintf(fh, "<H3> You have one new message.</H3>\n");
  else if (nm > 1) fprintf(fh, "<H3> You have %d new messages.</H3>\n", nm);

  // Just description list.
  fprintf(fh, "<dl>\n");

  // Scan the directory for the numeric files.
  while ((dent = readdir(dh)) != NULL)
  {
    if (isNumeric(dent->d_name))
    {
      // Get the message number
      n = atoi(dent->d_name);

      // Get its html name and set up the description list entry.
      sprintf(fname, "%d.html",n);
      fprintf(fh, "<dt> %d: </dt> <dd> <a href=\"%s\">", n, fname);

      // Now read the first 20 characters of the message as the content.
      sprintf(fname, "%s/%d", MailDir, n);
      if ((mfh = fopen(fname, "rb")) == NULL)
      {
        fprintf(stderr, "%s:%s:%d Couldn't open %s because %s\n", __FILE__, __func__, __LINE__,
                fname, strerror(errno));
        exit(-1);
      }
      for (i = 0; i < 20; ++i)
      {
        c = fgetc(mfh);
        if (c == EOF) break;
        fprintf(fh, "%c", c);
      }

      // Close out the description.
      fprintf(fh, "</a> ... </dd>\n");
      fclose(mfh);
    }
  }

  // Close the directory.
  closedir(dh);

  // Epilog on the HTML file.
  fprintf(fh, "</dl>\n</body>\n</html>\n");
  fclose(fh);
}



// EOF
