/* Copyright 2024 by COG9 Enterprises LLC
 * All rights reserved.
 * Title: listen.c
 * Author: Jed Marti
 * Description: Listen for a connection on the port number given. No timeout. Works on windows
 *   and Linux hence the ifdef's dotted about.
 * Created: Mon Dec  2 06:15:32 2024
 * Revision History:
 */

#include <stdio.h>
#include <stdint.h>
#include <errno.h>
#include <string.h>

#ifdef WINDOWS
#include <io.h>
#include <winsock.h>
#include <process.h>

// The maximum length of a host name.
#define MAXHOSTNAMELEN 255

// Why can't they be standard?
#define sleep _sleep

#else
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netdb.h>
#include <netinet/in.h>
#endif

// Global file handle. If -1, then not assigned yet.
int32_t FD = -1;


#ifdef WINDOWS
void startTCP(void)
/*     Windows requires this startup sequence, Linux is just a stub.
 * Parameters: none
 * External References: none
 * Returned values: none
 * Errors detected: System won't start.
 */
{
  WORD verReq;
  WSADATA wsdat;
  int fd;
  verReq = MAKEWORD(1, 1);
  if ((fd = WSAStartup(verReq, &wsdat)) != 0)
  {
    fprintf(stderr, "%s:%s:%d Couldn't start Windows networking\n", __FILE__, __func__, __LINE__);
    exit(-1);
  }
}
#endif
#ifdef LINUX
void startTCP(void)
{
}
#endif


void netListen(int32_t portnum)
/*     Listen on a port for a while and timeout if nobody calls it.
 * Parameters:
 *   portnum: 1024+ are acceptable, below that you must be super user.
 *   timeout: Number of seconds to wait before timing out.
 * External References:
 *   FD: file handle for I/O. -1 means to net connection.
 * Bugs: if the connection doesn't time out, the timeout may happen anyway?
 * Returnede value: none
 */
{
  static struct sockaddr_in my_addr;
  static struct sockaddr_in his_addr;
#ifdef WINDOWS
  static int64_t sock;
#endif
#ifdef LINUX
  static int32_t sock;
#endif
  uint32_t hisipv4;
  int32_t on;
  int32_t from_len;

  // Get a socket 
  sock = socket(AF_INET, SOCK_STREAM, 0);
  if (sock < 0)
  {
    fprintf(stderr, "%s:%s:%d No available sockets\n", __FILE__, __func__, __LINE__);
    exit(-1);
  }

  // Setup portnum`
  memset(&my_addr, 0, sizeof(my_addr));
  my_addr.sin_family = AF_INET;
  my_addr.sin_addr.s_addr = INADDR_ANY;

#ifdef WINDOWS
  my_addr.sin_port = htons((short) portnum);
#endif
#ifdef LINUX
  my_addr.sin_port = htons(portnum);
#endif
    
  // Get the port
  if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on)))
  {
    fprintf(stderr, "%s:%s:%d Can't reuse socket on port %d\n", __FILE__, __func__, __LINE__,
            portnum);
    exit(-1);
  }

#ifdef WINDOWS
  if (bind(sock, (struct sockaddr *) &my_addr, sizeof(my_addr)) < 0)
#endif
#ifdef LINUX
  if (bind(sock, (struct sockaddr *) &my_addr, sizeof(my_addr)) < 0 )
#endif
  {
    fprintf(stderr, "%s:%s:%d Can't bind to port number %d on this socket\n", __FILE__,
            __func__, __LINE__, portnum);
    exit(-1);
  }

  //  Do the listen
  if (listen(sock, 1) < 0)
  {
#ifdef WINDOWS
    fprintf(stderr, "%s:%s:%d Listen fails\n", __FILE__, __func__, __LINE__);
#endif
#ifdef LINUX
    fprintf(stderr, "%s:%s:%d Listen fails because %s\n", __FILE__, __func__, __LINE__,
            strerror(errno));
#endif
    exit(-1);
  }
  from_len = sizeof(his_addr);

#ifdef WINDOWS
     FD = accept(sock, (struct sockaddr *) &his_addr, &from_len);
#endif
#ifdef LINUX
     FD = accept(sock, (struct sockaddr *) &his_addr, (uint32_t *) &from_len);
#endif
  if (FD < 0)
  {
    fprintf(stderr, "%s:%s:%d Listen fails because %s\n", __FILE__, __func__, __LINE__,
            strerror(errno));
    exit(-1);
  }

  hisipv4 = his_addr.sin_addr.s_addr;
  printf("%s:%s:%d accept of %d.%d.%d.%d finishes with FD=%d\n", __FILE__,
	 __func__, __LINE__, hisipv4 >> 24, (hisipv4 >> 16) & 0xFF,
	 (hisipv4 >> 8) & 0xFF, hisipv4 & 0xFF, FD);


#ifdef WINDOWS
  closesocket(sock);
#endif
#ifdef LINUX
  close(sock);
#endif
}



// EOF
